type operator = Add | Sub | Mul | Div | Mod
              | Leq | Geq | Lthan | Gthan
              | Eq | Neq | And | Or

type uop = Neg | Not

type typ = Bool | Int | Char | String | Float | Void | Note | Duration | Tune | Array of typ

type bind = typ * string

type expr =
    IntLit of int
  | BoolLit of bool
  | StringLit of string
  | FloatLit of float
  | Binop of expr * operator * expr
  | Unop of uop * expr
  | Id of string
  | NoteLit of string
  | DurationLit of string
  | TuneLit of string
  | Call of string * expr list
  | Assign of typ * string * expr
  | Reassign of string * expr
  | ArrayLit of expr list
  | ArrayLitDefault of typ * expr
  | ArrayAccess of string * expr
  | ArrayElementAssign of string * expr * expr
  | Augment of expr * expr
  | Concat of expr * expr
  | Noexpr

type stmt =
    Block of stmt list
  | While of expr * stmt
  | Return of expr
  | If of expr * stmt * stmt
  | For of expr * expr * expr * stmt
  | Expr of expr
  | Nostmt

type func_decl = {
	typ : typ;
	fname : string;
	formals : bind list;
	body : stmt list;
}

type program = stmt list * func_decl list

let rec string_of_typ = function
    Int -> "int"
  | Bool -> "bool"
  | String -> "string"
  | Float -> "float"
  | Void -> "void"
  | Note -> "note"
  | Duration -> "duration"
  | Array t -> string_of_typ t ^ "[]"
  | Tune -> "tune"
  | Char -> "char"

let string_of_op = function
    Add -> "+"
  | Sub -> "-"
  | Mul -> "*"
  | Div -> "/"
  | Eq -> "=="
  | Neq -> "!="
  | Lthan -> "<"
  | Leq -> "<="
  | Gthan -> ">"
  | Geq -> ">="
  | And -> "&&"
  | Or -> "||"
  | Mod -> "%"

let string_of_uop = function
    Neg -> "-"
  | Not -> "!"

let rec string_of_expr = function
    StringLit(l) ->  l
  | IntLit(i) -> string_of_int i
  | BoolLit(true) -> "true"
  | BoolLit(false) -> "false"
  | FloatLit(f) -> string_of_float f
  | NoteLit(n) -> n
  | DurationLit(d) -> d
  | TuneLit(t) -> t
  | Id(s) -> s
  | Unop(o, e) -> string_of_uop o ^ string_of_expr e
  | Assign(t, v, e) -> string_of_typ t ^ " " ^ v ^ " = " ^ string_of_expr e
  | Call(f, el) ->
    f ^ "(" ^ String.concat ", " (List.map string_of_expr el) ^ ")"
  | Binop(e1, o, e2) ->
    string_of_expr e1 ^ " " ^ string_of_op o ^ " " ^ string_of_expr e2
  | Reassign(v, e) -> v ^ "=" ^ string_of_expr e
  | ArrayLit a -> "[" ^ String.concat " " (List.map string_of_expr a) ^ "]"
  | ArrayLitDefault(t, e) -> string_of_typ t ^ "[" ^ string_of_expr e ^ "]"
  | Noexpr -> ""
  | ArrayAccess(v, i) -> v ^ "[" ^ string_of_expr i ^ "]"
  | ArrayElementAssign(s, i, e) -> s ^ "[" ^ string_of_expr i ^ "]" ^ " = " ^ string_of_expr e
  | Augment(l, r) -> string_of_expr l ^ ":" ^ string_of_expr r
  | Concat(l, r) -> string_of_expr l ^ "." ^ string_of_expr r

let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(expr) -> string_of_expr expr ^ "\n";
  | If(e, s, Block([])) -> "if (" ^ string_of_expr e ^ ")\n" ^ string_of_stmt s
  | If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | While(e, s) -> "while (" ^ string_of_expr e ^ ") " ^ string_of_stmt s
  | Return(expr) -> "return " ^ string_of_expr expr ^ "\n"
  | For(e1, e2, e3, s) ->
      "for (" ^ string_of_expr e1  ^ " ; " ^ string_of_expr e2 ^ " ; " ^
      string_of_expr e3  ^ ") " ^ string_of_stmt s
  | Nostmt -> ""

let string_of_fdecl fdecl =
  string_of_typ fdecl.typ ^ " " ^
  fdecl.fname ^ "(" ^ String.concat ", " (List.map snd fdecl.formals) ^
  ")\n{\n" ^
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"

let string_of_program (funcs) =
  String.concat "\n" (List.map string_of_fdecl funcs)

