module L = Llvm
module A = Ast
module S = Semant

module StringMap = Map.Make(String)

let translate (functions) =

  (* Set LLVM Types *)
  let context = L.global_context () in
  let the_module = L.create_module context "sick-beets"
  and i32_t  = L.i32_type  context
  and i8_t   = L.i8_type   context
  and i1_t   = L.i1_type   context
  and f_t = L.double_type context
  and str_t = L.pointer_type (L.i8_type context)
  and note_t = L.pointer_type (L.i8_type context)
  and dur_t = L.pointer_type (L.i8_type context)
  and tune_t = L.pointer_type (L.i8_type context)
  and void_t = L.void_type context in

  (* Mapping for AST type to LLVM types *)
  let rec ltype_of_typ = function
      A.Int -> i32_t
    | A.Float -> f_t
    | A.String -> str_t
    | A.Note -> note_t
    | A.Tune -> tune_t
    | A.Duration -> dur_t
    | A.Char -> i8_t
    | A.Bool -> i1_t
    | A.Void -> void_t
    | A.Array t -> L.pointer_type (ltype_of_typ t) in

  (* Array creation, initialization, access *)
  let create_array t len builder =
    let ltype = ltype_of_typ t in
    let size_t = L.build_intcast (L.size_of ltype) i32_t "tmp" builder in
    let total_size = L.build_mul size_t len "tmp" builder in
    let total_size = L.build_add total_size (L.const_int i32_t 1) "tmp" builder in
    let arr_malloc = L.build_array_malloc ltype total_size "tmp" builder in
    let arr = L.build_pointercast arr_malloc (L.pointer_type ltype) "tmp" builder in
    arr
  in

  let initialize_array t el builder =
    let len = L.const_int i32_t (List.length el) in
    let arr = create_array t len builder in
    let _ =
      let assign_value i =
        let index = L.const_int i32_t i in
        let index = L.build_add index (L.const_int i32_t 1) "tmp" builder in
        let _val = L.build_gep arr [| index |] "tmp" builder in
        L.build_store (List.nth el i) _val builder
      in
      for i = 0 to (List.length el)-1 do
        ignore (assign_value i)
      done
    in
    arr
  in

  let access_array arr index assign builder =
    let index = L.build_add index (L.const_int i32_t 1) "tmp" builder in
    let arr = L.build_load arr "tmp" builder in
    let _val = L.build_gep arr [| index |] "tmp" builder in
    if assign then _val else L.build_load _val "tmp" builder
  in

  (* initialize global variable map from Ast.Id to llvalue *)
  let global_vars = ref StringMap.empty in

  (* Declare and define printf, strcat, strcpy *)
  let printf_t = L.var_arg_function_type i32_t [| L.pointer_type i8_t |] in
  let printf_func = L.declare_function "printf" printf_t the_module in

  let strcat_t = L.function_type (L.pointer_type i8_t) [| L.pointer_type i8_t; L.pointer_type i8_t |] in
  let strcat_func = L.declare_function "strcat" strcat_t the_module in

  let strcpy_t = L.function_type (L.pointer_type i8_t) [| L.pointer_type i8_t; L.pointer_type i8_t |] in
  let strcpy_func = L.declare_function "strcpy" strcpy_t the_module in

  (* create a map of function declarations *)
  let function_decls =
    let function_decl m fdecl =
      let name = fdecl.A.fname
      and formal_types =
        Array.of_list (List.map (fun (t,_) -> ltype_of_typ t) fdecl.A.formals) in
      let ftype = L.function_type (ltype_of_typ fdecl.A.typ) formal_types in
      StringMap.add name (L.define_function name ftype the_module, fdecl) m in
      List.fold_left function_decl StringMap.empty functions in

  (* Format str for printf *)
  let string_format_str b = L.build_global_stringptr "%s" "fmt" b
  and int_format_str    b = L.build_global_stringptr "%d" "fmt" b
  and float_format_str  b = L.build_global_stringptr "%f" "fmt" b in

  let format_str x_type builder =
    let b = builder in
      match x_type with
        A.Int      -> int_format_str b
      | A.Float    -> float_format_str b
      | A.String | A.Note | A.Duration | A.Tune  -> string_format_str b
      | A.Bool     -> int_format_str b
      | _ -> raise (Failure ("Invalid printf type"))
  in

  (* build the statements in a function body *)
  let build_function_body fdecl =
    let (f, _) = StringMap.find fdecl.A.fname function_decls in
    let builder = L.builder_at_end context (L.entry_block f) in

    (* function to add local variables to a map *)
    let local_vars =
      let add_formal m (t,n) p = L.set_value_name n p;
      let local = L.build_alloca (ltype_of_typ t) n builder in
      ignore (L.build_store p local builder);
      StringMap.add n (t,local) m in

      (* add formals to the map *)
      ref (List.fold_left2 add_formal StringMap.empty fdecl.A.formals
        (Array.to_list (L.params f))) in

    (* Return the value or the type for a variable or formal argument *)
    (* All the tables have the structure (type, llvalue) *)
    let name_to_llval n : L.llvalue =
      try (snd (StringMap.find n !local_vars))
      with Not_found -> (snd (StringMap.find n !global_vars))
    in

    let name_to_type n : A.typ =
      try (fst (StringMap.find n !local_vars))
      with Not_found -> (fst (StringMap.find n !global_vars)) in

    (* get type *)
    let rec gen_type = function
        A.IntLit _ -> A.Int
      | A.FloatLit _ -> A.Float
      | A.StringLit _ -> A.String
      | A.BoolLit _ -> A.Bool
      | A.ArrayLit el -> A.Array (gen_type (List.nth el 0))
      | A.ArrayLitDefault (t,_) -> t
      | A.ArrayElementAssign (_, _, el) -> gen_type el
      | A.NoteLit _ -> A.Note
      | A.DurationLit _ -> A.Duration
      | A.TuneLit _ -> A.Tune
      | A.Id name -> (match (name_to_type name) with
                      A.Array(t) -> t
                    | _ as ty -> ty)
      | A.Unop(_,e) -> gen_type e
      | A.Binop(e1,_,_) -> gen_type e1
      | A.Assign(_,var,_) -> gen_type (A.Id(var))
      | A.Reassign(var,_) -> gen_type (A.Id(var))
      | A.Call(var,_) -> let fdecl =
                            List.find (fun x -> x.A.fname = var) functions in
                            fdecl.A.typ
      | A.ArrayAccess(id,_) -> gen_type (A.Id(id))
      | A.Augment(_,_) |  A.Concat(_,_) -> A.Tune
      | A.Noexpr -> A.Void
    in

    (* get value of int *)
    let get_value_of_int = function
        A.IntLit x -> x
      | _ -> raise (Failure ("can only get_value_of_int of an int"))
    in

    (* Declare system functions we need *)
    let system_t = L.function_type i32_t [| str_t |] in
    let system_func = L.declare_function "system" system_t the_module in

    (* build string *)
    let build_string s n builder =
    let zero_const = L.const_int i32_t 0 in
      let str = L.build_in_bounds_gep s [| zero_const |] " " builder in
      L.build_call system_func [| str |] n builder
    in

    (* expression builder *)
    let rec expr builder = function
        A.IntLit i -> L.const_int i32_t i
      | A.StringLit s -> L.build_global_stringptr s "string" builder
      | A.FloatLit f -> L.const_float f_t f
      | A.BoolLit b -> L.const_int i1_t (if b then 1 else 0)
      | A.ArrayLit el -> let t = gen_type (List.nth el 0) in
        initialize_array t (List.map (expr builder) el) builder
      | A.ArrayLitDefault (t, e) -> let len = expr builder e in
        create_array t len builder
      | A.NoteLit n -> L.build_global_stringptr n "note" builder
      | A.DurationLit d -> L.build_global_stringptr d "duration" builder
      | A.TuneLit t -> L.build_global_stringptr t "tune" builder
      | A.Id s -> let llval = (name_to_llval s) in
                  L.build_load llval s builder
      | A.Noexpr -> L.const_int i32_t 0
      | A.Unop(op,e) -> let e' = expr builder e in
        (match op with
          A.Neg     -> L.build_neg
          | A.Not     -> L.build_not) e' "tmp" builder
      | A.Binop (e1, op, e2) ->
        let e1' = expr builder e1
        and e2' = expr builder e2
        and float_binops =
          match op with
            A.Add -> L.build_fadd
          | A.Sub -> L.build_fsub
          | A.Mul -> L.build_fmul
          | A.Div -> L.build_fdiv
          | A.Mod     -> L.build_frem
          | A.And     -> L.build_and
          | A.Or      -> L.build_or
          | A.Eq -> L.build_fcmp L.Fcmp.Oeq
          | A.Neq -> L.build_fcmp L.Fcmp.One
          | A.Lthan -> L.build_fcmp L.Fcmp.Ult
          | A.Leq -> L.build_fcmp L.Fcmp.Ole
          | A.Gthan -> L.build_fcmp L.Fcmp.Ogt
          | A.Geq -> L.build_fcmp L.Fcmp.Oge

        and int_binops =
          match op with
            A.Add -> L.build_add
          | A.Sub -> L.build_sub
          | A.Mul -> L.build_mul
          | A.Div -> L.build_sdiv
          | A.Mod     -> L.build_urem
          | A.And     -> L.build_and
          | A.Or      -> L.build_or
          | A.Eq -> L.build_icmp L.Icmp.Eq
          | A.Neq -> L.build_icmp L.Icmp.Ne
          | A.Lthan -> L.build_icmp L.Icmp.Slt
          | A.Leq -> L.build_icmp L.Icmp.Sle
          | A.Gthan -> L.build_icmp L.Icmp.Sgt
          | A.Geq -> L.build_icmp L.Icmp.Sge
        in

         if (L.type_of e1' = f_t || L.type_of e2' = f_t) then float_binops e1' e2' "tmp" builder
         else int_binops e1' e2' "tmp" builder

      | A.Call ("print", [e]) ->
          let e' = expr builder e in
          let e_type = gen_type e in
          L.build_call printf_func [| (format_str e_type builder) ; e' |] "printf" builder
      | A.Call ("render", [e1; e2]) ->
          let exec = expr builder (A.StringLit("CFugue/bin/testCFugueLib \""))
          and e1' = expr builder e1
          and quote = expr builder (A.StringLit("\" "))
          and e2' = expr builder e2 in
          let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
          let tmp1 = L.build_call strcpy_func [| buf; exec |] "strcpy" builder in
          let tmp2 = L.build_call strcat_func [| tmp1; e1' |] "strcat" builder in
          let tmp3 = L.build_call strcat_func [| tmp2; quote |] "strcat" builder in
          let render_cmd = L.build_call strcat_func [| tmp3; e2' |] "strcat" builder in
          build_string render_cmd "rendf" builder;
      | A.Call ("setInstrument", [e1; e2]) ->
          let tune = expr builder e1 and instr = expr builder e2 in
          let left = expr builder (A.StringLit("I[")) in
          let right = expr builder (A.StringLit("] ")) in
          let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
          let tmp1 = L.build_call strcpy_func [| buf; left |] "tmp1" builder in
          let tmp2 = L.build_call strcat_func [| tmp1; instr |] "tmp2" builder in
          let tmp3 = L.build_call strcat_func [| tmp2; right |] "tmp3" builder in
        L.build_call strcat_func [| tmp3; tune |] "strcat" builder
      | A.Call ("addLayer", [e1; e2; e3]) ->
          let ltune = expr builder e1 and rtune = expr builder e2 and layer = expr builder e3 in
          let channel = expr builder (A.StringLit("V")) in
          let space = expr builder (A.StringLit(" ")) in
          let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
          let tmp1 = L.build_call strcpy_func [| buf; ltune |] "tmp1" builder in
          let tmp2 = L.build_call strcat_func [| tmp1; channel |] "tmp2" builder in
          let tmp3 = L.build_call strcat_func [| tmp2; layer |] "tmp3" builder in
          let tmp4 = L.build_call strcat_func [| tmp3; space |] "tmp4" builder in
          L.build_call strcat_func [| tmp4; rtune |] "strcat" builder
      | A.Call ("setTempo", [e1; e2]) ->
          let tune' = expr builder e1
          and e' = expr builder (A.StringLit((string_of_int (get_value_of_int e2))))
          and space' = expr builder (A.StringLit(" "))
          and t' = expr builder (A.StringLit("T")) in
          let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
          let _ = L.build_call strcpy_func [| buf; t' |] "partial" builder in
          let _ = L.build_call strcat_func [| buf; e' |] "partial_2" builder in
          let _ = L.build_call strcat_func [| buf; space' |] "partial_3" builder in
          L.build_call strcat_func [| buf; tune' |] "strcat" builder
      | A.Call (f, act) ->
          let (fdef, fdecl) = StringMap.find f function_decls in
          let actuals = List.rev (List.map (expr builder) (List.rev act)) in
          let result = (match fdecl.A.typ with
             A.Void -> ""
            | _ -> f ^ "_result") in
          L.build_call fdef (Array.of_list actuals) result builder
      | A.Assign (t, s, e) ->
          let _ = (match fdecl.A.fname with
            "main" ->
              let init = (match t with
                A.Int   -> expr builder (A.IntLit(0))
              | A.Float -> expr builder (A.FloatLit(0.0))
              | A.String -> expr builder (A.StringLit(""))
              | A.Bool   -> expr builder (A.BoolLit(false))
              | A.Note -> expr builder (A.NoteLit(""))
              | A.Duration -> expr builder (A.DurationLit(""))
              | A.Tune -> expr builder (A.TuneLit(""))
              | t -> L.const_null (ltype_of_typ t)
            )
          in
                (global_vars := StringMap.add s (t, (L.define_global s init the_module)) !global_vars)
            | _ -> (local_vars := StringMap.add s (t, (L.build_alloca (ltype_of_typ t)) s builder) !local_vars)
            ) in
          let e' = expr builder e and llval = name_to_llval s in
          ignore (L.build_store e' llval builder); e'
      | A.Reassign (s, e) ->
          let e' = expr builder e and llval = name_to_llval s in
          ignore (L.build_store e' llval builder); e'
      | A.ArrayAccess (s, i) ->
          let index = expr builder i and llval = name_to_llval s in
          access_array llval index false builder
      | A.ArrayElementAssign (s, i, e) ->
          let e' = expr builder e in
          let index = expr builder i in
          let llval = name_to_llval s in
          let var = access_array llval index true builder in
          ignore (L.build_store e' var builder); e'
      | A.Augment(e1, e2) ->
          let space = expr builder (A.StringLit(" ")) in
          let empty = expr builder (A.StringLit("")) in

          (match (e1,e2) with
            (A.NoteLit _, A.DurationLit _) ->
              let note = expr builder e1 and dur = expr builder e2 in
              let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
              let space = expr builder (A.StringLit(" ")) in
              let _ = L.build_call strcpy_func [| buf; note |] "tmp1" builder in
              let _ = L.build_call strcat_func [| buf; dur |] "tmp2" builder in
              L.build_call strcat_func [| buf; space|] "strcat" builder

            | (A.NoteLit _, A.ArrayLit a) ->
              let x = expr builder e1 in
              let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
              let buf = L.build_call strcpy_func [| buf; empty |] "tmp" builder in
              let _ = List.fold_left (fun s e ->
                let e' = expr builder e in
                let _ = L.build_call strcat_func[| s; x|] "tmp2" builder in
                let _ = L.build_call strcat_func[| s; e'|] "tmp3" builder in
                L.build_call strcat_func [| s; space |] "strcat" builder ) buf a  in
              L.build_call strcat_func [| buf; space |] "strcat" builder

            | (A.ArrayLit a, A.DurationLit _) ->
              let x = expr builder e2 in
              let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
              let buf = L.build_call strcpy_func [| buf; empty |] "tmp" builder in
              let _ = List.fold_left (fun s e ->
                let e' = expr builder e in
                let _ = L.build_call strcat_func[| s; e'|] "tmp2" builder in
                let _ = L.build_call strcat_func[| s; x|] "tmp3" builder in
                L.build_call strcat_func [| s; space |] "strcat" builder ) buf a  in
              L.build_call strcat_func [| buf; space |] "strcat" builder

            | (A.ArrayLit n, A.ArrayLit d) ->
              let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
              let buf = L.build_call strcpy_func [| buf; empty |] "tmp" builder in  
              let _ = List.fold_left2 (fun s e1 e2 ->
                let e1' = expr builder e1 in
                let e2' = expr builder e2 in
                let _ = L.build_call strcat_func[| s; e1'|] "tmp2" builder in
                let _ = L.build_call strcat_func[| s; e2'|] "tmp3" builder in
                L.build_call strcat_func [| s; space |] "strcat" builder ) buf n d  in
              L.build_call strcat_func [| buf; space |] "strcat" builder

            | (A.Id _, A.Id _) | (A.NoteLit _, A.Id _) | (A.Id _, A.DurationLit _) ->
              let e1' = expr builder e1 and e2' = expr builder e2 in
              let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
              let tmp = L.build_call strcpy_func [| buf; e1'|] "tmp" builder in

              L.build_call strcat_func [| tmp; e2'|] "strcat" builder
            | (_, _) -> expr builder (A.TuneLit(""))
          )

      | A.Concat(e1, e2) ->
          let e1' = expr builder e1 and e2' = expr builder e2 in
          let space = expr builder (A.StringLit(" ")) in
          let buf = expr builder (A.ArrayLitDefault(A.Char, A.IntLit(4096))) in
          let tmp1 = L.build_call strcpy_func [| buf; e1' |] "tmp1" builder in
          let tmp2 = L.build_call strcat_func [| tmp1; e2' |] "tmp2" builder in
          L.build_call strcat_func [|tmp2;space |] "strcat" builder
      in

    (* Invoke "f builder" if the current block doesn't already
       have a terminal (e.g., a branch). *)
    let add_terminal builder' f' =
      match L.block_terminator (L.insertion_block builder') with
          Some _ -> ()
        | None -> ignore(f' builder') in

    (* Build the code for the given statement; return the builder for
       the statement's successor *)
    let rec stmt builder = function
        A.Block sl -> List.fold_left stmt builder sl
      | A.Expr e -> ignore(expr builder e); builder
      | A.Nostmt -> ignore (0); builder
      | A.Return e -> ignore (match fdecl.A.typ with
         A.Void -> L.build_ret_void builder
         | _ -> L.build_ret (expr builder e) builder); builder
      | A.If (predicate, then_stmt, else_stmt) ->
        let bool_val = expr builder predicate in
        let merge_bb = L.append_block context "merge" f in

        let then_bb = L.append_block context "then" f in
        add_terminal (stmt (L.builder_at_end context then_bb) then_stmt)
         (L.build_br merge_bb);

        let else_bb = L.append_block context "else" f in
        add_terminal (stmt (L.builder_at_end context else_bb) else_stmt)
          (L.build_br merge_bb);

        ignore (L.build_cond_br bool_val then_bb else_bb builder);
        L.builder_at_end context merge_bb

      | A.While (predicate, body) ->
        let pred_bb = L.append_block context "while" f in
        ignore (L.build_br pred_bb builder);

        let body_bb = L.append_block context "while_body" f in
        add_terminal (stmt (L.builder_at_end context body_bb) body)
          (L.build_br pred_bb);

        let pred_builder = L.builder_at_end context pred_bb in
        let bool_val = expr pred_builder predicate in

        let merge_bb = L.append_block context "merge" f in
        ignore (L.build_cond_br bool_val body_bb merge_bb pred_builder);
        L.builder_at_end context merge_bb
      | A.For (e1, e2, e3, body) -> stmt builder
        ( A.Block [A.Expr e1; A.While (e2, A.Block [body; A.Expr e3]) ] )
    in

    (* Build the code for all statements in the function *)
    let builder = stmt builder (A.Block fdecl.A.body) in

    (* Add a return if the last block falls off the end *)
    add_terminal builder (match fdecl.A.typ with
        A.Int -> L.build_ret (L.const_int i32_t 0)
      | A.Float -> L.build_ret (L.const_float f_t 0.0)
      | A.Bool -> L.build_ret (L.const_int i1_t 0)
      | A.String | A.Note | A.Duration | A.Tune -> L.build_ret (L.const_int i8_t 0)
      | _ -> L.build_ret_void) in

  List.iter build_function_body functions ;

  the_module
