%{ open Ast %}

%token LPAREN RPAREN LBRACE RBRACE
%token PLUS MINUS TIMES DIVIDE ASSIGN MODULUS
%token LESS GREATER LEQ GEQ EQ NEQ AND OR EOF NOT NEG COLON SEMI DOT
%token RETURN WHILE IF ELIF ELSE FOR
%token EOL
%token BOOL INT STRING FLOAT VOID NOTE DURATION TUNE CHAR
%token <int> INTLIT
%token <string> STRINGLIT
%token <float> FLOATLIT
%token <string> ID
%token <string> NOTELIT
%token <string> DURATIONLIT
%token <string> TUNELIT
%token TRUE FALSE
%token LBRACKET RBRACKET COMMA

%nonassoc NOELSE
%nonassoc ELSE

%right ASSIGN REASSIGN
%nonassoc LBRACKET RBRACKET
%left OR
%left AND
%left LESS GREATER LEQ GEQ
%left EQ NEQ
%left PLUS MINUS
%left TIMES DIVIDE MODULUS
%left DOT
%left AT
%left COLON
%left DOLLAR
%right NOT NEG
%nonassoc RETURN
%nonassoc IF

%start program
%type <Ast.program> program


%%


program: decls EOF { $1 }

decls:
  /* nothing */   { [], [] }
| decls stmt      { (fst $1)@[$2], snd $1 }
| decls func_decl { fst $1, ($2 :: snd $1) }

stmts:
  /* nothing  */  { [] }
| stmts stmt      { $2 :: $1 }

stmt:
| WHILE LPAREN expr RPAREN stmt         { While($3, $5) }
| LBRACE stmts RBRACE EOL               { Block(List.rev $2) }
| RETURN expr EOL                       { Return $2 }
| IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
| IF LPAREN expr RPAREN stmt ELSE stmt  { If($3, $5, $7) }
| FOR LPAREN expr SEMI expr SEMI expr RPAREN stmt { For($3, $5, $7, $9) }
| expr EOL                              { Expr($1) }
| EOL                                   { Nostmt }


expr:
  INTLIT        { IntLit($1) }
| FLOATLIT      { FloatLit($1) }
| STRINGLIT     { StringLit($1) }
| TRUE          { BoolLit(true) }
| FALSE         { BoolLit(false) }
| ID            { Id($1) }
| NOTELIT       { NoteLit($1) }
| DURATIONLIT   { DurationLit($1) }
| TUNELIT       { TuneLit($1) }
| expr PLUS expr    { Binop($1, Add, $3) }
| expr DOT expr     { Concat($1, $3) }
| expr MINUS expr   { Binop($1, Sub, $3) }
| expr COLON expr   { Augment($1, $3) }
| expr TIMES expr   { Binop($1, Mul, $3) }
| expr DIVIDE expr  { Binop($1, Div, $3) }
| expr MODULUS expr { Binop($1, Mod, $3) }
| expr LESS expr    { Binop($1, Lthan, $3) }
| expr GREATER expr { Binop($1, Gthan, $3) }
| expr LEQ expr     { Binop($1, Leq, $3) }
| expr GEQ expr     { Binop($1, Geq, $3) }
| expr EQ expr      { Binop($1, Eq, $3) }
| expr NEQ expr     { Binop($1, Neq, $3) }
| expr AND expr     { Binop($1, And, $3) }
| expr OR expr      { Binop($1, Or, $3) }
| MINUS expr %prec NEG  { Unop(Neg, $2) }
| NOT expr              { Unop(Not, $2) }
| typ ID ASSIGN expr    { Assign($1, $2, $4) }
| ID ASSIGN expr        { Reassign($1, $3) }
| ID LPAREN actuals_opt RPAREN          { Call($1, $3) }
| LBRACKET elements_opt RBRACKET        { ArrayLit($2) }
| primitive LBRACKET expr RBRACKET      { ArrayLitDefault($1, $3) }
| ID LBRACKET expr RBRACKET             { ArrayAccess($1, $3) }
| ID LBRACKET expr RBRACKET ASSIGN expr { ArrayElementAssign($1, $3, $6) }

typ:
  primitive                   { $1 }
| primitive LBRACKET RBRACKET { Array($1) }

primitive:
  INT       { Int }
| FLOAT     { Float }
| BOOL      { Bool }
| STRING    { String }
| VOID      { Void }
| NOTE      { Note }
| DURATION  { Duration }
| TUNE      { Tune }
| CHAR      { Char}

actuals_opt:
  /* nothing*/  { [] }
| actuals_list  { List.rev $1 }

actuals_list:
  expr                    { [$1] }
| actuals_list COMMA expr { $3 :: $1 }

func_decl:
  typ ID LPAREN formals_opt RPAREN LBRACE stmts RBRACE
  {{  typ = $1;
      fname = $2;
      formals = $4;
      body = List.rev $7 }}

formals_opt:
   /*nothing*/  { [] }
| formal_list   { List.rev $1 }

formal_list:
  typ ID                   { [($1, $2)] }
| formal_list COMMA typ ID { ($3, $4) :: $1 }

elements_opt:
  /*nothing*/   { [] }
| elements_list { List.rev $1 }

elements_list:
  expr               { [$1] }
| elements_list COMMA expr { $3 :: $1 }

