(* Semantic checking for the sick-beets compiler *)

open Ast

module StringMap = Map.Make(String)

(* Semantic checking of a program. Returns void if successful,
   throws an exception if something is wrong.
   Check each global variable, then check each function *)

let check (functions) =

  (* Helper: Raises exception if given list has duplicate *)
  let report_duplicate exceptf list =
    let rec helper = function
      n1 :: n2 :: _ when n1 = n2 -> raise (Failure (exceptf n1))
      | _ :: t -> helper t
      | [] -> ()
    in helper (List.sort compare list)
  in

  (* Helper: Returns value type or raises exception if cannot be assigned *)
  let check_assign ltyp rtyp err =
    if ltyp = rtyp then ltyp else raise err
  in

  (**** Checking Functions ****)

  (* Helper: Check banned function decl *)
  let check_func_decl func_name =
    if List.mem func_name (List.map (fun fd -> fd.fname) functions)
  then raise (Failure ("function cannot be named " ^ func_name))
  in

  check_func_decl "print";
  check_func_decl "render";
  check_func_decl "addLayer";
  check_func_decl "setTempo";
  check_func_decl "setInstrument";

  (* Check for duplicated function declarations *)
  report_duplicate (fun n -> "duplicate function [" ^ n ^ "]")
  (List.map (fun fd -> fd.fname) functions);

  (* Built in function declarations *)
  let built_in_decls = StringMap.add "render" { typ = Void; fname = "render"; formals = [(Tune, "x"); (String, "name")]; body = [] }
  (StringMap.singleton "print" { typ = Void; fname = "print"; formals = [(String, "x")]; body = [] })
  in

  let built_in_decls = StringMap.add "setInstrument" { typ = Tune; fname = "setInstrument"; formals = [(Tune, "x"); (String, "instrument")]; body = [] } built_in_decls in
  let built_in_decls = StringMap.add "setTempo" { typ = Tune; fname = "setTempo"; formals = [(Tune, "x"); (Int, "tempo")]; body = [] } built_in_decls in
  let built_in_decls = StringMap.add "addLayer" { typ = Tune; fname = "addLayer"; formals = [(Tune, "x"); (Tune, "y"); (String, "layer")]; body = []} built_in_decls in

  (* Accepted types for built in functions *)
  let print_types = [String; Int; Bool; Float; Duration; Note; Tune] in
  let render_types = [String; Tune] in
  let addLayer_types = [String; Tune; Tune] in
  let setinstrument_types = [Tune; String] in

  (* Stores all function declarations *)
  let function_decls = List.fold_left (fun m fd -> StringMap.add fd.fname fd m)
    built_in_decls functions
  in

  (* Helper: Check if input function was declared *)
  let function_decl s = try StringMap.find s function_decls
    with Not_found -> raise (Failure ("unrecognized function [" ^ s ^ "]"))
  in

  (* Store global variables from Ast.Id to llvalue *)
  let global_vars = ref StringMap.empty in

  (* Helper:Semant check input function *)
  let check_function func =
    (* Check for duplicate formals *)
    report_duplicate (fun n -> "duplicate formal [" ^ n ^ "] in function " ^ func.fname)
      (List.map snd func.formals);

    (* Store type and names of variables - formal *)
    let variables = ref (List.fold_left (fun m (t, n) -> StringMap.add n t m)
    StringMap.empty (func.formals))
    in

    (* Helper: Returns type of identifier *)
    let type_of_identifier name =
      try StringMap.find name (!variables)
      with Not_found -> try StringMap.find name (!global_vars)
      with Not_found -> raise (Failure ("undeclared identifier " ^ name))
    in

    (* Helper: Check if variable is already declared *)
    let check_var_decl var_name err =
      if StringMap.mem var_name (!variables) || StringMap.mem var_name (!global_vars)
      then raise err
    in

    let array_typ = function
      Array typ -> typ
      | _ -> raise(Failure("Expecting an array and was not an array"))
    in

    let typ_of_lit = function
        IntLit _ -> Int
      | FloatLit _ -> Float
      | StringLit _ -> String
      | BoolLit _ -> Bool
      | NoteLit _ -> Note
      | DurationLit _ -> Duration
      | TuneLit _ -> Tune
      | _ -> raise(Failure("Expecting a literal."))
    in

    (* Helper: Returns type of expression or exception *)
    let rec expr = function
        IntLit _ -> Int
      | FloatLit _ -> Float
      | StringLit _ -> String
      | BoolLit _ -> Bool
      | NoteLit _ -> Note
      | Id name -> type_of_identifier name
      | DurationLit _ -> Duration
      | TuneLit _ -> Tune
      | Noexpr -> Void
      | ArrayLit(el) -> Array (typ_of_lit (List.nth el 0))
      | ArrayLitDefault (t, _) -> Array t
      | ArrayAccess(var , el) as element->
        if expr el != Int then raise (Failure ("Invalid element access in " ^ string_of_expr element))
        else array_typ (type_of_identifier var)
      | Unop (op, e) as ex -> let t = expr e in
        (match op with
          Neg when t = Int -> Int
          | Not when t = Bool -> Bool
          | _ -> raise(Failure("illegal unary operation in " ^ string_of_expr ex)))
      | Binop(e1, op, e2) as e ->
          let t1 = expr e1 and t2 = expr e2 in
          (match op with
              Add when t1=Tune && t2 = Tune -> Tune
            | Add | Sub | Mul | Div | Mod when t1 = Int && t2 = Int -> Int
            | Add | Sub | Mul | Div | Mod when t1 = Float && t2 = Float -> Float
            | Eq | Neq when t1 = t2 -> Bool
            | Lthan | Leq | Gthan | Geq when t1 = Int && t2 = Int -> Bool
            | Lthan | Leq | Gthan | Geq when t1 = Float && t2 = Float -> Bool
            | And | Or when t1 = Bool && t2 = Bool -> Bool
            | _ -> raise(Failure ("illegal binary operator " ^
              string_of_typ t1 ^ " " ^ string_of_op op ^ " " ^
              string_of_typ t2 ^ " in " ^ string_of_expr e ))
          )
      | Augment(e1, e2) ->
        let t1 = expr e1 and t2 = expr e2 in
        (match (t1, t2) with
        (Note, Duration)|(Note, Array(Duration)) | (Array(Note),Duration) |
          (Array(Note), Array(Duration)) -> Tune
          |(_,_) -> raise(Failure("illegal augment " ^ string_of_typ t1 ^ " : " ^ string_of_typ t2 ^ ". must be of note(lit or array) : duration(lit or array")))
      | Concat(e1, e2) ->
        let t1 = expr e1 and t2 = expr e2 in
        if t1 = Tune && t2 = Tune then Tune
        else raise(Failure ("illegal concatenation, " ^ string_of_typ t1 ^ " and " ^ string_of_typ t2 ^ " not both Tunes."))
      | Assign(typ, var, e) as ex ->
          let rt = expr e in
          ignore (check_assign typ rt (Failure ("illegal assignment " ^ string_of_typ typ ^ " = " ^ string_of_typ rt ^ " in " ^ string_of_expr ex)));
          check_var_decl var (Failure ("duplicate declaration of variable " ^ var));
          let _ =
            (match func.fname with
              "main" -> global_vars := StringMap.add var typ (!global_vars)
              | _ -> variables := StringMap.add var typ (!variables)
            )
          in rt
      | Reassign(var, e) as ex ->
          let rt = expr e and lt = type_of_identifier var in
          check_assign lt rt (Failure ("illegal assignment " ^ string_of_typ lt ^ " = " ^ string_of_typ rt ^ " in " ^ string_of_expr ex))
      | ArrayElementAssign (s, i, e) as ex ->
          let lt =
          if expr i != Int then raise (Failure ("invalid element access in " ^ string_of_expr ex))
          else array_typ (type_of_identifier s) in
          let rt = expr e in
          check_assign lt rt (Failure ("illegal assignment " ^ string_of_typ lt ^ " = " ^ string_of_typ rt ^ " in " ^ string_of_expr ex));
      | Call(fname, actuals) as call ->
          let fd = function_decl fname in
          if List.length actuals != List.length fd.formals then
            raise (Failure ("expecting " ^ string_of_int (List.length fd.formals) ^
              " arguments in " ^ string_of_expr call))
          else
            let _ =
              (match fname with
                "print" ->
                  ignore (List.iter (fun e ->
                    let etyp = expr e in
                    if (List.mem etyp print_types) == false then
                      raise (Failure ("illegal actual argument found " ^ string_of_typ etyp ^ " in " ^ string_of_expr e))) actuals);
                | "render" ->
                  ignore (List.iter (fun e ->
                    let etyp = expr e in
                    if (List.mem etyp render_types) == false then
                      raise (Failure ("illegal actual argument found " ^ string_of_typ etyp ^ " in " ^ string_of_expr e))) actuals);
                    if (List.length actuals != 2) then
                      raise (Failure ("render needs two arguments"));
                | "addLayer" ->
                  ignore (List.iter (fun e ->
                    let etyp = expr e in
                    if (List.mem etyp addLayer_types) == false then
                      raise (Failure ("Illegal actual argument found " ^ string_of_typ etyp ^ " in " ^ string_of_expr e))) actuals);
                    if (List.length actuals != 3) then
                      raise (Failure ("addLayer needs three arguments"));
                | "setInstrument" ->
                  if (List.length actuals != 2) then
                      raise (Failure ("setInstrument needs two arguments"));
                  List.iter2 (fun ftyp e ->
                    let etyp = expr e in
                    ignore (check_assign ftyp etyp (Failure ("illegal actual argument found " ^ string_of_typ etyp ^ " expected " ^ string_of_typ ftyp ^ " in " ^ string_of_expr e)))
                  ) setinstrument_types actuals
                | _ ->
                  List.iter2 (fun (ftyp, _) e ->
                    let etyp = expr e in
                    ignore (check_assign ftyp etyp (Failure ("illegal actual argument found " ^ string_of_typ etyp ^ " expected " ^ string_of_typ ftyp ^ " in " ^ string_of_expr e)))
                  ) fd.formals actuals
              ) in
            fd.typ
    in

    (* Helper: Check input boolean expression *)
    let check_bool_expr e = if expr e != Bool then
      raise (Failure ("Expected boolean expression in " ^ string_of_expr e))
    else () in

    (* Helper: Check input statement *)
   let rec stmt = function
      Block sl ->
        let rec check_block = function
          [Return _ as s] -> stmt s
        | Return _ :: _ -> raise (Failure "unreached code after return")
        | Block sl :: ss -> check_block (sl @ ss)
        | s :: ss -> stmt s; check_block ss
        | [] -> ()
        in check_block sl
      | Expr e -> ignore (expr e)
      | Return e -> let t = expr e in if t = func.typ then () else
        raise (Failure ("Returns type " ^ string_of_typ t ^ " instead of expected type " ^ string_of_expr e ))
      | If(e, b1, b2) -> check_bool_expr e; stmt b1; stmt b2
      | While(e, b) -> check_bool_expr e; stmt b
      | For(e1, e2, e3, st) ->
        ignore (expr e1); check_bool_expr e2; ignore(expr e3); stmt st
      | Nostmt -> ignore (0)
      (* for *)
    in

    (* Check statements in function block *)
    stmt (Block func.body)
  in

  List.iter check_function functions


