#!/bin/sh

# Regression testing script for MicroC
# Step through a list of files
#  Compile, run, and check the output of each expected-to-work test
#  Compile and check the error of each expected-to-fail test

# Path to the LLVM interpreter
LLI="lli"
#LLI="/usr/local/opt/llvm/bin/lli"

# Path to the microc compiler.  Usually "./microc.native"
# Try "_build/microc.native" if ocamlbuild was unable to create a symbolic link.
MICROC="./sb.native"
#MICROC="_build/microc.native"

# Set time limit for all operations
ulimit -t 30

globallog=testall.log
errtype=".err"
rm -f $globallog
error=0
globalerror=0

keep=0
passed=0
inc=1
failed=0
total=0

Usage() {
    echo "Usage: testall.sh [options] [.sb files]"
    echo "-k    Keep intermediate files"
    echo "-h    Print this help"
    exit 1
}

SignalError() {
    if [ $error -eq 0 ] ; then
	echo "FAILED"
	error=1
    fi
    echo "  $1"
}

# Compare <outfile> <reffile> <difffile>
# Compares the outfile with reffile.  Differences, if any, written to difffile
Compare() {
    generatedfiles="$generatedfiles $3"
    echo diff -b $1 $2 ">" $3 1>&2
    diff -b "$1" "$2" > "$3" 2>&1 || {
	SignalError "$1 differs"
	echo "FAILED $1 differs from $2" 1>&2
    }
}

# Run <args>
# Report the command, run it, and report any errors
Run() {
    # echo $* 1>&2
    eval $* || {
	SignalError "$1 failed on $*"
	return 1
    }
}

# RunFail <args>
# Report the command, run it, and expect an error
RunFail() {
    echo $* 1>&2
    eval $* && {
	SignalError "failed: $* did not report an error"
	return 1
    }
    return 0
}

Check() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.sb//'`
    reffile=`echo $1 | sed 's/.sb$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    generatedfiles="$generatedfiles ${basename}.ll ${basename}.out ${basename}.err" &&
    Run "$MICROC" "<" $1 ">" "${basename}.ll" "2>" "${basename}.err" &&
    Run "$LLI" "${basename}.ll" ">" "${basename}.out" &&
    Compare ${basename}.out ${reffile}.out ${basename}.diff

    # Run "$MICROC" < $1 2> "${basename}.err" 1> "${basename}.dumpc"
    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
	if [ $keep -eq 0 ] ; then
	    rm -f $generatedfiles
	fi
	echo "OK"
	echo "###### SUCCESS" 1>&2
    (( passed += inc ))
    else
	echo "###### FAILED" 1>&2
    (( failed += inc ))
    printf "\e[1;31m $basename \n\e[0m" >> failedtests.out
    echo | cat "$basename"".err" >> failedtests.out
    echo | cat "$basename"".diff" >> failedtests.out
	globalerror=$error
    fi
}

CheckFail() {
    error=0
    basename=`echo $1 | sed 's/.*\\///
                             s/.sb//'`
    reffile=`echo $1 | sed 's/.sb$//'`
    basedir="`echo $1 | sed 's/\/[^\/]*$//'`/."

    echo -n "$basename..."

    echo 1>&2
    echo "###### Testing $basename" 1>&2

    generatedfiles=""

    generatedfiles="$generatedfiles ${basename}.err ${basename}.diff" &&
    RunFail "$MICROC" "<" $1 "2>" "${basename}.err" ">>" $globallog &&
    Compare ${basename}.err ${reffile}.err ${basename}.diff

    # Report the status and clean up the generated files

    if [ $error -eq 0 ] ; then
	if [ $keep -eq 0 ] ; then
	    rm -f $generatedfiles
	fi
	echo "OK"
	echo "###### SUCCESS" 1>&2
    (( passed += inc ))
    else
	echo "###### FAILED" 1>&2
    (( failed += inc ))
    printf "\e[1;31m $basename \n\e[0m" >> failedtests.out
    echo | cat "$basename"".err" >> failedtests.out
    echo | cat "$basename"".diff" >> failedtests.out
	globalerror=$error
    fi
}

while getopts kdpsh c; do
    case $c in
	k) # Keep intermediate files
	    keep=1
	    ;;
	h) # Help
	    Usage
	    ;;
    esac
done

shift `expr $OPTIND - 1`

LLIFail() {
  echo "Could not find the LLVM interpreter \"$LLI\"."
  echo "Check your LLVM installation and/or modify the LLI variable in testall.sh"
  exit 1
}

which "$LLI" >> $globallog || LLIFail


if [ $# -ge 1 ]
then
    files=$@
else
    files="tests/test-*.sb tests/fail-*.sb"
fi

for file in $files
do
    case $file in
	*test-*)
	    Check $file 2>> $globallog
	    ;;
	*fail-*)
	    CheckFail $file 2>> $globallog
	    ;;
	*)
	    echo "unknown file type $file"
	    globalerror=1
	    ;;
    esac
done


((total += passed + failed))

echo | awk '{for (i=1; i<=75; i++) printf "-"; printf "\n"}'
echo | awk -v passed="$passed" -v failed="$failed" '{printf "\033[1;32m";
for (i=1; i<=passed; i++) printf "+"; printf "\033[0m "; printf "\033[1;31m";
for (i=1; i<=failed; i++) printf  "-"; printf "\033[0m "; printf "\n"}'
echo $passed out of $total tests passed
echo
echo Failing Tests:
echo | cat failedtests.out
echo | awk '{for (i=1; i<=75; i++) printf "-"; printf "\n"}'
> failedtests.out


rm
exit $globalerror
